// Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2465 $
// $LastChangedDate:: 2022-02-15 15:12:58 +0100#$
//
// Compares major-, minor- and maintainance version against current Scilab version and returns false, 
// if current Scilab version is less than comparing version.
// Otherwise it returns true.

function [isVerLessThan] = isVersionLessThan(compareVersion)
    versionIdSize = 3;
    
    curVersion = getversion('scilab');
    cmpVersionParts = strsplit(compareVersion, ".");
    if size(cmpVersionParts, 'r') ~= versionIdSize then
        error("Invalid version string");
    end
    for i=1:versionIdSize
        cmpVersion(i) = evstr(cmpVersionParts(i));
    end
    
    isVerLessThan = %f;
    for i=1:versionIdSize
        if curVersion(i) < cmpVersion(i) then
            isVerLessThan = %t;
            break;
        elseif curVersion(i) > cmpVersion(i) then
            isVerLessThan = %f;
            break;
        end
    end
    
endfunction

function [is_major_ver_less_than] = isMajorVersionLessThan(cmp_major_ver)
    scilab_ver = getversion('scilab');
    scilab_major_ver = scilab_ver(1);
    
    if scilab_major_ver < cmp_major_ver then
        is_major_ver_less_than = %t;
    else
        is_major_ver_less_than = %f;
    end
endfunction
